<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

require_once 'midgard_setup_config.php';
require_once 'midgard_setup_database.php';
require_once 'midgard_setup_pear.php';
require_once 'midgard_setup_vhost.php';
require_once 'midgard_setup_log.php';
require_once 'midgard_admin_sitewizard_creator_sitegroup.php';

/**
 * Base class for all midgard setup types.
 * Setup is divided into single steps and each of them performs
 * two tasks: configure and install.
 * 
 * Configure part is designed to ask user and get particular data, 
 * so midgard_quick_setup minimize this part and tries to collect 
 * sensible defaults. However any class which extends this one, may 
 * ask user its own way; through dialog windows, shell inputs, or 
 * html or gtk widgets.
 * 
 * Install part is designed to perform installation without asking 
 * the user. One may compare it to debian's 'preinst' and 'postinst'
 * scripts. In this part derived class should be aware what is able 
 * to do. For example www user interface can not correctly install
 * Virtual hosts, however may configure it.
 */

class midgard_quick_setup
{
    /**
     * midgard_setup_config object which is responsible to initialize 
     * correct midgard_setup_ui object and holds reference to named 
     * configuration through midgard_config object. The latter may 
     * be initialized without named configuration if needed.
     */
    protected $setup_config;

    /**
     * midgard_setup_database object
     */
    protected $midgard_setup_database;

    /**
     * midgard_setup_pear object
     */
    protected $midgard_setup_pear;

    /**
     * midgard_setup_vhost object
     */
    protected $midgard_setup_vhost;

    /**
     * Object which is responsible for host creation.
     * By default it's midgard_admin_sitewizard_creator_host.
     */
    protected $host_creator = null;

    /**
     * Object which is responsible for sitegroup creation.
     * By default it's midgard_admin_sitewizard_creator_sitegroup.
     */
    protected $sitegroup_creator = null;

    /**
     * An array which holds all sitegroup related data:
     * 'object' MidgardSitegroup object ( will be replaced with midgard_sitegroup )
     * 'admin' name of sitegroup administrator
     * 'password' password of sitegroup administrator
     */
    protected $sitegroup = array(); 

    /**
     * Named midgard configuration used by instance
     */
    protected $config_name;

    /**
     * Name of the host
     */
    protected $host_name = null;

    /**
     * Port of the host
     */
    protected $host_port = 80;
    
    /**
     * Prefix of the host
     */
    protected $host_prefix = "";

    /**
     * midgard_config object, rarely initialized
     * The one associated with setup_config should be used instead.
     */
    public $midgard_config;

    /**
     * Initialize new midgard_setup_config and midgard_setup_ui_cli objects
     */
    public function __construct($name = NULL)
    {
        if($name != NULL && $name != "")
        {
            $this->setup_config = new midgard_setup_config("cli", $name);
            
        } else {
        
            $this->config_name = "midgard";
            $midgard_config = new midgard_config();
            $this->setup_config =
                new midgard_setup_config("cli", NULL, &$midgard_config);
        }

        $this->sitegroup["object"] = null;
        $this->sitegroup["admin"] = "admin";
        $this->sitegroup["password"] = "password";
    }

    final public function factory($setup_name)
    {
        /* TODO ? */
    }

    /**
     * Configure pear packages.
     * Quick setup by default install all packages
     */
    public function configure_pear()
    {
        $this->setup_config->ui->message(_("Configure all packages in quick setup type"));
    }

    /**
     * Install configured pear packages
     */
    public function install_pear()
    {
        midgard_setup_log::write(_("Installing PEAR packages"));

        try {
        
            $this->midgard_setup_pear = new midgard_setup_pear();

        } catch (Exception $e) {

            $this->setup_config->ui->error($e->getMessage());
        }
        $this->midgard_setup_pear->set_channels();
        $this->midgard_setup_pear->install_midcom_roles();
        $this->midgard_setup_pear->install_base_packages();
    }

    /**
     * Install configured midgard database
     * This method invokes shell commands ( like mysql ), 
     * so it's not recommended to invoke it in web browser interface.
     */
    final public function install_database()
    {
        try {
            
            $this->midgard_setup_database = 
                new midgard_setup_database(&$this->setup_config);
        
        } catch (Exception $e) {
            
            $this->setup_config->ui->error($e->getMessage());
        }
        
        $this->midgard_setup_database->set_admin_name("root");
        $this->midgard_setup_database->create_database_blobdir();
        $this->midgard_setup_database->create_database(); 
    }

    /**
     * Updates database
     * It's safe to call this method on any level as database update
     * is performed with Midgard API. There's no need to initialize
     * any connection, or invoke any commands ( like mysql ).
     */
    public function update_database()
    {
        if(isset($this->midgard_setup_database))
            return $this->midgard_setup_database->update_database();

        try {
            
            $this->midgard_setup_database = 
                new midgard_setup_database(&$this->setup_config);
        
        } catch (Exception $e) {
            
            $this->setup_config->ui->error($e->getMessage());
        }

        $this->midgard_setup_database->update_database();
    }

    public function set_vhost($name, $port = 80, $prefix = "")
    {
        $this->host_name = $name;
        $this->host_port = $port;
        $this->host_prefix = $prefix;
    }

    public function configure_sitegroup()
    {
        $this->setup_config->ui->message(_("Configure default sitegroup in quick setup type"));
    }

    public function install_sitegroup()
    {
        /* Try to create sitegroup */
        
        /* Log in with configuration's credentials */
        $root_name = $this->setup_config->midgard_config->midgardusername;
        $root_pass = $this->setup_config->midgard_config->midgardpassword;
        $rootid = mgd_auth_midgard($root_name, $root_pass);
        if($rootid == 0) 
        {
            $this->setup_config->ui->error(_("Failed to log in as root.")." ID: 0");
        
        } else {
            
            $this->setup_config->ui->message(_("Logged in as root")."(${rootid})");
        }

        /* FIXME, it's workaround, we need to get sitegroup name 
        * before we ask for hostname */
        if($this->host_name == null)
            $this->host_name = exec("hostname");

        $default_hostname = $this->host_name;

        $sg_creator = 
            new midgard_admin_sitewizard_creator_sitegroup(&$this->setup_config);
        $this->sitegroup_creator = &$sg_creator;
        $sg_creator->set_sitegroup_name($this->host_name);
        $sg_creator->set_sitegroup_admin_group_name($this->host_name." administrators");
                
        $created = $sg_creator->execute();

        if($created)
        {
            $this->setup_config->ui->message(_("Succesfully created sitegroup") . " ( {$default_hostname} )");

        } else {
            
            $this->setup_config->ui->error(_("Couldn't create sitegroup") . "( {$default_hostname} )");
        }

        /* sitegroup is created, let's log in as sitegroup administrator 
         * to create initial host */
        $sg_creator->login_administrator();   
    }

    public function configure_vhost()
    {
        $this->setup_config->ui->message(_("Configure Apache virtual host"));   

        /* TODO, *any* error handling here is needed */
        if($this->host_name == null)
            $default_hostname = exec("hostname");

        if(!isset($this->host_name))
        {
            $value = $this->setup_config->ui->get_text(_("Hostname to use ?"), $default_hostname);
            
            if(!$value && $value == '') 
            {
                $value = $default_hostname;
            
            } else {
                
                $this->setup_config->ui->warning(_("Can not create host with empty name"));
                $this->configure_vhost();
            }

            $this->set_vhost($value);
        }

        $this->host_creator = 
            new midgard_admin_sitewizard_creator_host(&$this->setup_config);
        /* FIXME, change to Hemingway or Gray template */
        $this->host_creator->set_host_style("Template_SimpleGray"); 
    }

    public function install_vhost()
    {
        $this->setup_config->ui->message(_("Installing Apache virtual host"));

        if(is_null($this->host_creator))
            $this->host_creator = 
                new midgard_admin_sitewizard_creator_host(&$this->setup_config);
        
        $this->host_creator->set_host_url($this->host_name);
        $this->host_creator->set_sitegroup($this->sitegroup_creator->get_sitegroup());
        $this->host_creator->execute();

        try 
        {
            $this->midgard_setup_vhost =
                new midgard_setup_vhost(&$this->setup_config,
                $this->host_name, $this->host_port);
                
        } catch (Exception $e) {
        
            $this->setup_config->ui->error($e->getMessage());
        }

        $this->midgard_setup_vhost->set_host_directories();
        $this->midgard_setup_vhost->enable_apache_module();
        $this->midgard_setup_vhost->create_configuration();
        
        /* FIXME, derived class might want to ask before reloading */
        $this->midgard_setup_vhost->apache_reload();
    }
}
?>
